@@ -2,19 +2,17 @@
    Copyright (C) 2001-2002 Hewlett-Packard Co
 	Contributed by David Mosberger-Tang <davidm@hpl.hp.com>
 
-This library is free software; you can redistribute it and/or modify
-it under the terms of the GNU Lesser General Public License as
-published by the Free Software Foundation; either version 2.1 of the
-License, or (at your option) any later version.
+This file is part of libunwind.
 
-This library is distributed in the hope that it will be useful, but
-WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-Lesser General Public License for more details.
+libunwind is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2, or (at your option)
+any later version.
 
-You should have received a copy of the GNU Lesser General Public
-License along with this library; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+libunwind is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
 
 As a special exception, if you link this library with other files to
 produce an executable, this library does not by itself cause the
@@ -23,8 +21,7 @@ This exception does not however invalidate any other reasons why the
 executable file might be covered by the GNU General Public
 License.  */
 
-#include <ucontext.h>
-#include <unwind.h>
+#include <libunwind.h>
 
 /* See glibc manual for a description of this function.  */
 
@@ -32,11 +29,11 @@ int
 backtrace (void **buffer, int size)
 {
   unw_cursor_t cursor;
-  ucontext_t uc;
+  unw_context_t uc;
   unw_word_t ip;
   int n = 0;
 
-  getcontext (&uc);
+  unw_getcontext (&uc);
   if (unw_init_local (&cursor, &uc) < 0)
     return 0;
 
