@@ -2,19 +2,17 @@
    Copyright (C) 2001-2002 Hewlett-Packard Co
 	Contributed by David Mosberger-Tang <davidm@hpl.hp.com>
 
-This library is free software; you can redistribute it and/or modify
-it under the terms of the GNU Lesser General Public License as
-published by the Free Software Foundation; either version 2.1 of the
-License, or (at your option) any later version.
+This file is part of libunwind.
 
-This library is distributed in the hope that it will be useful, but
-WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-Lesser General Public License for more details.
+libunwind is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2, or (at your option)
+any later version.
 
-You should have received a copy of the GNU Lesser General Public
-License along with this library; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+libunwind is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
 
 As a special exception, if you link this library with other files to
 produce an executable, this library does not by itself cause the
@@ -274,8 +272,7 @@ desc_abi (unsigned char abi, unsigned char context,
   if (abi == 0 && context == 's')
     sr->flags |= IA64_FLAG_SIGTRAMP;
   else
-    dprintf ("unwind: ignoring unwabi(abi=0x%x,context=0x%x)\n", abi,
-	     context);
+    dprintf ("unwind: ignoring unwabi(abi=0x%x,context=0x%x)\n", abi, context);
 }
 
 static inline void
@@ -665,25 +662,17 @@ lookup (struct ia64_unwind_table *table, unw_word_t rel_ip)
   return e;
 }
 
-int
-__ia64_unw_create_state_record (struct ia64_cursor *c,
-				struct ia64_state_record *sr)
+static int
+get_proc_info (struct ia64_cursor *c)
 {
   const struct ia64_unwind_table_entry *e = 0;
-  unw_word_t ip = c->ip, predicates = c->pr;
   struct ia64_unwind_table *table;
-  unw_ia64_table_t info;
-  struct ia64_reg_info *r;
+  unw_word_t segbase, len;
   uint8_t *dp, *desc_end;
+  unw_ia64_table_t info;
+  unw_word_t ip = c->ip;
   uint64_t hdr;
-  STAT(unsigned long start;)
-  STAT(++unw.stat.parse.calls; start = ia64_get_itc ());
-
-  /* build state record */
-  memset (sr, 0, sizeof (*sr));
-  for (r = sr->curr.reg; r < sr->curr.reg + IA64_NUM_PREGS; ++r)
-    r->when = IA64_WHEN_NEVER;
-  sr->pr_val = predicates;
+  int ret;
 
   /* search the kernels and the modules' unwind tables for IP: */
 
@@ -691,15 +680,20 @@ __ia64_unw_create_state_record (struct ia64_cursor *c,
     if (ip >= table->start && ip < table->end)
       break;
 
-  if (!table && ia64_acquire_unwind_info (c, ip, &info) >= 0)
+  if (!table)
     {
-      unw_word_t segbase = info.segbase;
-      unw_word_t len = info.length;
+      ret = ia64_acquire_unwind_info (c, ip, &info);
+      if (ret < 0)
+	return ret;
 
+      segbase = info.segbase;
+      len = info.length;
+
+      /* XXX avoid malloc: */
       table = malloc (sizeof (struct ia64_unwind_table));
       if (!table)
 	{
-	  dprintf (__FUNCTION__": out of memory\n");
+	  dprintf ("%s: out of memory\n", __FUNCTION__);
 	  return -1;
 	}
       memset (table, 0, sizeof (*table));
@@ -711,14 +705,50 @@ __ia64_unw_create_state_record (struct ia64_cursor *c,
       table->next = unw.tables;
       unw.tables = table;
       /* XXX LOCK } */
-
-      assert (ip >= table->start && ip < table->end);
     }
 
-  if (table)
-    e = lookup (table, ip - table->info.segbase);
+  assert (ip >= table->start && ip < table->end);
 
-  if (!e)
+  e = lookup (table, ip - table->info.segbase);
+
+  hdr = *(uint64_t *) (table->info.unwind_info_base + e->info_offset);
+  dp = (uint8_t *) (table->info.unwind_info_base + e->info_offset + 8);
+  desc_end = dp + 8 * IA64_UNW_LENGTH (hdr);
+
+  c->pi.flags = 0;
+  c->pi.gp = table->info.gp;
+  c->pi.proc_start = table->info.segbase + e->start_offset;
+  c->pi.pers_addr = (uint64_t *) desc_end;
+  c->pi.desc = dp;
+
+  /* XXX Perhaps check UNW_VER / UNW_FLAG_OSMASK ? */
+  if (IA64_UNW_FLAG_EHANDLER (hdr) | IA64_UNW_FLAG_UHANDLER (hdr))
+    c->pi.flags |= IA64_FLAG_HAS_HANDLER;
+
+  return 0;
+}
+
+int
+ia64_create_state_record (struct ia64_cursor *c, struct ia64_state_record *sr)
+{
+  unw_word_t ip = c->ip, predicates = c->pr;
+  struct ia64_reg_info *r;
+  uint8_t *dp, *desc_end;
+  int ret;
+  STAT(unsigned long start;)
+  STAT(++unw.stat.parse.calls; start = ia64_get_itc ());
+
+  /* build state record */
+  memset (sr, 0, sizeof (*sr));
+  for (r = sr->curr.reg; r < sr->curr.reg + IA64_NUM_PREGS; ++r)
+    r->when = IA64_WHEN_NEVER;
+  sr->pr_val = predicates;
+
+  ret = get_proc_info (c);
+  if (ret < 0)
+    return ret;
+
+  if (!c->pi.desc)
     {
       /* No info, return default unwinder (leaf proc, no mem stack, no
          saved regs).  */
@@ -730,17 +760,16 @@ __ia64_unw_create_state_record (struct ia64_cursor *c,
       return 0;
     }
 
-  sr->proc_start = table->info.segbase + e->start_offset;
-  sr->when_target = 3 * ((ip & ~0xfUL) - sr->proc_start) / 16 + (ip & 0xfUL);
-
-  hdr = *(uint64_t *) (table->info.unwind_info_base + e->info_offset);
-  dp = (uint8_t *) (table->info.unwind_info_base + e->info_offset + 8);
-  desc_end = dp + 8 * IA64_UNW_LENGTH (hdr);
-  sr->pers_addr = (uint64_t *) desc_end;
+  sr->when_target = (3 * ((ip & ~0xfUL) - c->pi.proc_start)
+		     / 16 + (ip & 0xfUL));
 
+  dp = c->pi.desc;
+  desc_end = (uint8_t *) c->pi.pers_addr;
   while (!sr->done && dp < desc_end)
     dp = unw_decode (dp, sr->in_body, sr);
 
+  c->pi.flags |= sr->flags;
+
   if (sr->when_target > sr->epilogue_start)
     {
       /* sp has been restored and all values on the memory stack below
@@ -771,7 +800,7 @@ __ia64_unw_create_state_record (struct ia64_cursor *c,
   if (unw.debug_level > 0)
     {
       printf ("unwind: state record for func 0x%lx, t=%u:\n",
-	      table->info.segbase + e->start_offset, sr->when_target);
+	      c->pi.proc_start, sr->when_target);
       for (r = sr->curr.reg; r < sr->curr.reg + IA64_NUM_PREGS; ++r)
 	{
 	  if (r->where != IA64_WHERE_NONE || r->when != IA64_WHEN_NEVER)
@@ -810,7 +839,7 @@ __ia64_unw_create_state_record (struct ia64_cursor *c,
 }
 
 int
-__ia64_unw_free_state_record (struct ia64_state_record *sr)
+ia64_free_state_record (struct ia64_state_record *sr)
 {
   struct ia64_reg_state *rs, *next;
 
@@ -829,14 +858,18 @@ __ia64_unw_free_state_record (struct ia64_state_record *sr)
   return 0;
 }
 
-unw_word_t
-__ia64_get_gp (unw_word_t ip)
+int
+ia64_get_proc_info (struct ia64_cursor *c)
 {
-  struct ia64_unwind_table *table;
+#ifdef IA64_UNW_SCRIPT_CACHE
+  struct ia64_script *script = ia64_script_lookup (c);
 
-  for (table = unw.tables; table; table = table->next)
-    if (ip >= table->start && ip < table->end)
-      return table->info.gp;
+  if (script)
+    {
+      c->pi = script->pi;
+      return 0;
+    }
+#endif
 
-  return 0;
+  return get_proc_info (c);
 }
