@@ -2,19 +2,17 @@
    Copyright (C) 2001-2002 Hewlett-Packard Co
 	Contributed by David Mosberger-Tang <davidm@hpl.hp.com>
 
-This library is free software; you can redistribute it and/or modify
-it under the terms of the GNU Lesser General Public License as
-published by the Free Software Foundation; either version 2.1 of the
-License, or (at your option) any later version.
+This file is part of libunwind.
 
-This library is distributed in the hope that it will be useful, but
-WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-Lesser General Public License for more details.
+libunwind is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2, or (at your option)
+any later version.
 
-You should have received a copy of the GNU Lesser General Public
-License along with this library; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+libunwind is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
 
 As a special exception, if you link this library with other files to
 produce an executable, this library does not by itself cause the
@@ -27,8 +25,7 @@ License.  */
 
 #include <memory.h>
 #include <stdint.h>
-#include <unwind.h>
-#include <unwind-ia64.h>
+#include <libunwind.h>
 
 #define struct_offset(str,fld)	((char *)&((str *)NULL)->fld - (char *) 0)
 
@@ -51,7 +48,7 @@ License.  */
 # define dprintf(format...) \
 	printf (format)
 # define inline	__attribute__ ((unused))
-  extern const char *ia64_regname (unw_regnum_t reg);
+  extern const char *_U_ia64_regname (unw_regnum_t reg);
 #else
 # define debug(level,format...)
 # define dprintf(format...)
@@ -94,10 +91,16 @@ enum ia64_pregnum
 
 #define IA64_FLAG_SIGTRAMP	(1 << 0)	/* signal trampoline? */
 #define IA64_FLAG_BIG_ENDIAN	(1 << 1)	/* big-endian? */
+#define IA64_FLAG_HAS_HANDLER	(1 << 2)	/* has personality routine? */
 
-#ifdef IA64_UNW_SCRIPT_CACHE
-# include "script.h"
-#endif
+struct ia64_proc_info
+  {
+    unsigned int flags;		/* see IA64_FLAG_* above */
+    unw_word_t gp;		/* global pointer value */
+    unw_word_t proc_start;	/* start address of procedure */
+    uint64_t *pers_addr;	/* address of personality routine pointer */
+    uint8_t *desc;		/* encoded unwind descriptors (or NULL) */
+  };
 
 struct ia64_cursor
   {
@@ -112,11 +115,10 @@ struct ia64_cursor
     unw_word_t sp;		/* stack pointer value */
     unw_word_t psp;		/* previous sp value */
     unw_word_t ip;		/* instruction pointer value */
+    struct ia64_proc_info pi;	/* info about current procedure */
     unw_word_t pr;		/* current predicate values */
     unw_word_t cfm_loc;		/* cfm save location (or NULL) */
 
-    unw_word_t proc_start;	/* start address of procedure */
-    uint64_t *pers_addr;	/* address of personality routine pointer */
     unw_word_t rbs_top;		/* address of end of register backing store */
     unw_word_t top_rnat_loc;	/* location of final (topmost) RNaT word */
 
@@ -136,7 +138,7 @@ struct ia64_cursor
     unw_word_t b1_loc, b2_loc, b3_loc, b4_loc, b5_loc;
     unw_word_t f2_loc, f3_loc, f4_loc, f5_loc, fr_loc[16];
 
-    unsigned int flags;		/* IA64_FLAG_SIGTRAMP... */
+    unw_word_t eh_args[4];	/* exception handler arguments */
 
 #ifdef IA64_UNW_SCRIPT_CACHE
     short hint;
@@ -144,6 +146,10 @@ struct ia64_cursor
 #endif
 };
 
+#ifdef IA64_UNW_SCRIPT_CACHE
+# include "script.h"
+#endif
+
 /* Bits 0 to 2 of an location are used to encode its type:
 
 	bit 0: set if location uses floating-point format.
@@ -157,6 +163,7 @@ struct ia64_cursor
 #define IA64_LOC(l,t)		(((l) << 3) | (t))
 
 #define IA64_GET_LOC(l)		((l) >> 3)
+#define IA64_MASK_LOC_TYPE(l)	((l) & ~0x7)
 #define IA64_IS_FP_LOC(loc)	(((loc) & IA64_LOC_TYPE_FP) != 0)
 #define IA64_IS_MEMSTK_NAT(loc)	(((loc) & IA64_LOC_TYPE_MEMSTK_NAT) != 0)
 #define IA64_IS_REG_LOC(r)	(((r) & IA64_LOC_TYPE_REG) != 0)
@@ -174,9 +181,9 @@ struct ia64_cursor
 
 #ifdef IA64_UNW_ACCESSORS
 # define ia64_acquire_unwind_info(c,ip,i) \
-	(*c->acc.acquire_unwind_info)((ip), (i))
+	(*(c)->acc.acquire_unwind_info)((ip), (i), (c)->acc.arg)
 # define ia64_release_unwind_info(c,ip,i) \
-	(*c->acc.release_unwind_info)((i))
+	(*(c)->acc.release_unwind_info)((i), (c)->acc.arg)
 
 static inline int
 ia64_getfp (struct ia64_cursor *c, unw_word_t loc, unw_fpreg_t *val)
@@ -186,6 +193,7 @@ ia64_getfp (struct ia64_cursor *c, unw_word_t loc, unw_fpreg_t *val)
   if (IA64_IS_REG_LOC (loc))
     return (*c->acc.access_fpreg) (IA64_GET_LOC (loc), val, 0, c->acc.arg);
 
+  loc = IA64_MASK_LOC_TYPE(loc);
   ret = (*c->acc.access_mem) (loc + 0, &val->raw.bits[0], 0, c->acc.arg);
   if (ret < 0)
     return ret;
@@ -201,6 +209,7 @@ ia64_putfp (struct ia64_cursor *c, unw_word_t loc, unw_fpreg_t val)
   if (IA64_IS_REG_LOC (loc))
     return (*c->acc.access_fpreg) (IA64_GET_LOC (loc), &val, 1, c->acc.arg);
 
+  loc = IA64_MASK_LOC_TYPE(loc);
   ret = (*c->acc.access_mem) (loc + 0, &val.raw.bits[0], 1, c->acc.arg);
   if (ret < 0)
     return ret;
@@ -225,7 +234,7 @@ ia64_get (struct ia64_cursor *c, unw_word_t loc, unw_word_t *val)
       if (ret < 0)
 	return ret;
 
-      if (c->flags & IA64_FLAG_BIG_ENDIAN)
+      if (c->pi.flags & IA64_FLAG_BIG_ENDIAN)
 	*val = tmp.raw.bits[1];
       else
 	*val = tmp.raw.bits[0];
@@ -246,7 +255,7 @@ ia64_put (struct ia64_cursor *c, unw_word_t loc, unw_word_t val)
       unw_fpreg_t tmp;
 
       memset (&tmp, 0, sizeof (tmp));
-      if (c->flags & IA64_FLAG_BIG_ENDIAN)
+      if (c->pi.flags & IA64_FLAG_BIG_ENDIAN)
 	tmp.raw.bits[1] = val;
       else
 	tmp.raw.bits[0] = val;
@@ -260,12 +269,12 @@ ia64_put (struct ia64_cursor *c, unw_word_t loc, unw_word_t val)
 }
 
 #else
-  extern int __ia64_acquire_unwind_info (unw_word_t, void *);
-  extern int __ia64_release_unwind_info (void *);
-# define ia64_acquire_unwind_info(c,ip,i) \
-	__ia64_acquire_unwind_info((ip), (i))
-# define ia64_release_unwind_info(c,ip,i) \
-	__ia64_release_unwind_info((i))
+  extern int ia64_acquire_unwind_info (unw_word_t, void *);
+  extern int ia64_release_unwind_info (void *);
+# define ia64_acquire_unwind_info(c,ip,i)	\
+	ia64_acquire_unwind_info((ip), (i))
+# define ia64_release_unwind_info(c,ip,i)	\
+	ia64_release_unwind_info((i))
 # define ia64_get(c,l,v)	(*(v) = *(unw_word_t *) (l), 0)
 #endif
 
@@ -324,13 +333,11 @@ struct ia64_state_record
     unsigned int done : 1;		/* are we done scanning descriptors? */
     unsigned int any_spills : 1;	/* got any register spills? */
     unsigned int in_body : 1;		/* are we inside prologue or body? */
-    unsigned long flags;		/* see IA64_FLAG_* below */
+    unsigned int flags;		/* see IA64_FLAG_* */
 
     uint8_t *imask;		/* imask of of spill_mask record or NULL */
     unw_word_t pr_val;		/* predicate values */
     unw_word_t pr_mask;		/* predicate mask */
-    unw_word_t proc_start;	/* start address of procedure */
-    uint64_t *pers_addr;	/* address of personality routine pointer */
 
     long spill_offset;		/* psp-relative offset for spill base */
     int region_start;
@@ -422,26 +429,49 @@ struct ia64_global_unwind_state
 # endif /* IA64_UNW_STATS */
   };
 
-#define unw	__ia64_unw_data
+/* Convenience macros: */
+#define unw				UNW_OBJ(ia64_data)
+#define ia64_get_proc_info		UNW_OBJ(ia64_get_proc_info)
+#define ia64_create_state_record	UNW_OBJ(ia64_create_state_record)
+#define ia64_free_state_record		UNW_OBJ(ia64_free_state_record)
+#define ia64_get_frame_state		UNW_OBJ(ia64_get_frame_state)
+#define ia64_find_save_locs		UNW_OBJ(ia64_find_save_locs)
+#define ia64_init			UNW_OBJ(ia64_init)
+#define ia64_init_remote		UNW_OBJ(ia64_init_remote)
+#define ia64_glibc_acquire_unwind_info	UNW_OBJ(ia64_glibc_acquire_unwind_info)
+#define ia64_glibc_release_unwind_info	UNW_OBJ(ia64_glibc_release_unwind_info)
+#define ia64_access_reg			UNW_OBJ(ia64_access_reg)
+#define ia64_access_fpreg		UNW_OBJ(ia64_access_fpreg)
+#define ia64_get_sigcontext_addr	UNW_OBJ(ia64_get_sigcontext_addr)
 
 extern struct ia64_global_unwind_state unw;
 
-extern int __ia64_unw_create_state_record (struct ia64_cursor *c,
-					   struct ia64_state_record *sr);
-extern int __ia64_unw_free_state_record (struct ia64_state_record *sr);
-extern unw_word_t __ia64_get_gp (unw_word_t ip);
-extern int __ia64_unw_find_save_locs (struct ia64_cursor *c);
-extern void __ia64_unw_init (void);
-extern int __ia64_unw_init_remote (unw_cursor_t *c, unw_accessors_t *a);
-extern int __ia64_glibc_acquire_unwind_info (unw_word_t ip, void *info);
-extern int __ia64_glibc_release_unwind_info (void *info);
-extern int __ia64_access_reg (struct ia64_cursor *c, unw_regnum_t reg,
-			      unw_word_t *valp, int write);
-extern unw_word_t __ia64_get_sigcontext_addr (struct ia64_cursor *c);
+extern int ia64_get_proc_info (struct ia64_cursor *c);
+extern int ia64_create_state_record (struct ia64_cursor *c,
+				     struct ia64_state_record *sr);
+extern int ia64_free_state_record (struct ia64_state_record *sr);
+extern int ia64_get_frame_state (struct ia64_cursor *c);
+extern int ia64_find_save_locs (struct ia64_cursor *c);
+extern void ia64_init (void);
+extern int ia64_init_remote (unw_cursor_t *c, unw_accessors_t *a);
+extern int ia64_glibc_acquire_unwind_info (unw_word_t ip, void *info,
+					   void *arg);
+extern int ia64_glibc_release_unwind_info (void *info, void *arg);
+extern int ia64_access_reg (struct ia64_cursor *c, unw_regnum_t reg,
+			    unw_word_t *valp, int write);
+extern int ia64_access_fpreg (struct ia64_cursor *c, unw_regnum_t reg,
+			      unw_fpreg_t *valp, int write);
+extern unw_word_t ia64_get_sigcontext_addr (struct ia64_cursor *c);
+
+extern void __ia64_install_context (const ucontext_t *ucp, long r15, long r16,
+				    long r17, long r18)
+		   __attribute__ ((noreturn));
 
 /* XXX temporary (from glibc): */
 #define weak_alias(name, aliasname) \
   extern __typeof (name) aliasname __attribute__ ((weak, alias (#name)));
+#define alias(name, aliasname) \
+  extern __typeof (name) aliasname __attribute__ ((alias (#name)));
 
 /* XXX should be in glibc: */
 #ifndef IA64_SC_FLAG_ONSTACK
